@@ -8,6 +8,7 @@ commands:
 
     -iteralign (iterative mapping)
     -digest (genome chunking)
+    -cutsite (preprocess fastq by cutting reads into digestion products)
     -filter (Hi-C 'event' sorting: loops, uncuts, weird
      and 'true contacts')
     -view (map visualization)
@@ -49,6 +50,7 @@ import copy
 from Bio import SeqIO
 import hicstuff.view as hcv
 import hicstuff.hicstuff as hcs
+import hicstuff.cutsite as hcc
 import hicstuff.digest as hcd
 import hicstuff.iteralign as hci
 import hicstuff.filter as hcf
@@ -169,7 +171,7 @@ class Digest(AbstractCommand):
     named "info_contigs.txt" and "fragments_list.txt"
 
     usage:
-        digest [--plot] [--figdir=FILE] [--force] [--circular] [--size=INT]
+        digest [--plot] [--figdir=FILE] [--force] [--circular] [--size=0]
                [--outdir=DIR] --enzyme=ENZ <fasta>
 
     arguments:
@@ -237,6 +239,76 @@ class Digest(AbstractCommand):
         )
 
 
+class Cutsite(AbstractCommand):
+    """Cutsite command
+
+    Generates new gzipped fastq files from original fastq. The function will cut
+    the reads at their religation sites and creates new pairs of reads with the
+    different fragments obtained after cutting at the digestion sites.
+
+    There are three choices to how combine the fragments. 1. "for_vs_rev": All
+    the combinations are made between one forward fragment and one reverse
+    fragment. 2. "all": All 2-combinations are made. 3. "pile": Only
+    combinations between adjacent fragments in the initial reads are made.
+
+    usage:
+        cutsite --forward=FILE --reverse=FILE  --prefix=STR --enzyme=STR
+        [--threads=1] [--mode=for_vs_rev]
+
+    options:
+        -1, --forward=FILE      Fastq file containing the forward reads to 
+                                digest.
+        -2, --reverse=FILE      Fastq file containing the reverse reads to 
+                                digest.
+        -p, --prefix=STR        Prefix of the path where to write the digested 
+                                gzipped fastq files. Filenames will be added the
+                                suffix "_{1,2}.fq.gz".
+        -e, --enzyme=STR        The list of restriction enzyme used to digest
+                                the genome separated by a comma. Example:
+                                DpnII,HinfI.
+        -m, --mode=STR          Digestion mode. There are three possibilities:
+                                "for_vs_rev", "all" and "pile". The first one
+                                "for_vs_rev" makes all possible contact between
+                                fragments from forward read versus the fragments
+                                of the reverse reads. The second one "all"
+                                consist two make all pairs of fragments 
+                                possible. The third one "pile" will make the
+                                contacts only with the adjacent fragments.
+                                [Default: for_vs_rev]
+        -t, --threads=INT       Number of parallel threads allocated for the
+                                alignement. [Default: 1]
+    """
+
+    def execute(self):
+
+        # Check for mandatory options
+        for option in ["--prefix", "--forward", "--reverse"]:
+            if self.args[option] is None:
+                raise ValueError(f"{option} is mandatory.")
+
+        prefix = self.args["--prefix"]
+        # Create output directory if it does not exist
+        if dirname(prefix):
+            os.makedirs(dirname(prefix), exist_ok=True)
+        output_for = prefix + "_1.fq.gz"
+        output_rev = prefix + "_2.fq.gz"
+        # Digestion of the reads.
+        logger.info("Digestion of the reads:")
+        logger.info("Enzyme used: {0}".format(self.args["--enzyme"]))
+        logger.info(
+            "Mode used to cut the reads: {0}".format(self.args["--mode"])
+        )
+        hcc.cut_ligation_sites(
+            self.args["--forward"],
+            self.args["--reverse"],
+            output_for,
+            output_rev,
+            enzyme=self.args["--enzyme"],
+            mode=self.args["--mode"],
+            n_cpu=int(self.args["--threads"]),
+        )
+
+
 class Filter(AbstractCommand):
     """Mapping event filtering command
 
@@ -319,9 +391,9 @@ class View(AbstractCommand):
     output image to disk. If no output is specified, the output is displayed.
 
     usage:
-        view [--binning=1] [--despeckle] [--frags FILE] [--trim INT] [--n-mad FLOAT]
-             [--normalize] [--min=0] [--max=99%] [--output=IMG] [--cmap=CMAP] [--dpi=INT]
-             [--transform=FUN] [--circular] [--region=STR] <contact_map> [<contact_map2>]
+        view [--binning=1] [--despeckle] [--frags FILE] [--trim INT] [--n-mad 3.0] [--lines]
+             [--normalize] [--min=0] [--max=99%] [--output=IMG] [--cmap=Reds] [--dpi=300]
+             [--transform=STR] [--circular] [--region=STR] <contact_map> [<contact_map2>]
 
     arguments:
         contact_map             Sparse contact matrix in bg2, cool or graal format
@@ -334,21 +406,23 @@ class View(AbstractCommand):
         -b, --binning=INT[bp|kb|Mb|Gb]   Rebin the matrix. If no unit is given, bins will
                                          be merged by groups of INT. If a unit is given,
                                          bins of that size will be generated. [default: 1]
-        -c, --cmap=CMAP                  The name of a matplotlib colormap to
+        -c, --cmap=STR                   The name of a matplotlib colormap to
                                          use for the matrix. [default: Reds]
         -C, --circular                   Use if the genome is circular.
         -d, --despeckle                  Remove sharp increases in long range
                                          contact by averaging surrounding
                                          values.
         -D, --dpi=INT                    Map resolution in DPI (dots per inch). [default: 300]
-        -f, --frags=FILE                 Required for bp binning. Tab-separated
-                                         file with headers, containing
+        -f, --frags=FILE                 Required for bp binning and chromosome lines.
+                                         Tab-separated file with headers, containing
                                          fragments start position in the 3rd
                                          column, as generated by hicstuff
                                          pipeline.
-        -T, --transform=FUN              Apply a mathematical transformation to pixel values 
+        -T, --transform=STR              Apply a mathematical transformation to pixel values
                                          to improve visibility of long range signals. Possible
                                          values are: log2, log10, ln, sqrt, exp0.2.
+        -l, --lines                      Add dotted lines marking separation between chromosomes
+                                         or contigs. Requires --frags.
         -M, --max=INT                    Saturation threshold. Maximum pixel
                                          value is set to this number. Can be
                                          followed by % to use a percentile of
@@ -358,10 +432,10 @@ class View(AbstractCommand):
                                          identically to --max. [default: 0]
         -N, --n-mad=INT                  Number of median absolute deviations (MAD) from the median
                                          of log bin sums allowed to keep bins in the normalization
-                                         procedure [default: 3].
+                                         procedure [default: 3.0].
         -n, --normalize                  Should ICE normalization be performed
                                          before rendering the matrix ?
-        -o, --output=IMG                 Name of the image file where the view is stored.
+        -o, --output=FILE                Name of the image file where the view is stored.
         -r, --region=STR[;STR]           Only view a region of the contact map.
                                          Regions are specified as UCSC strings.
                                          (e.g.:chr1:1000-12000). If only one
@@ -406,9 +480,38 @@ class View(AbstractCommand):
         if self.binning > 1:
             if self.bp_unit:
                 self.pos = self.frags.iloc[:, 2]
-                binned_map, binned_frags = hcs.bin_bp_sparse(
+                binned_map, binned_pos = hcs.bin_bp_sparse(
                     M=sparse_map, positions=self.pos, bin_len=self.binning
                 )
+                # Get bin numbers of chromosome starts
+                binned_start = np.append(
+                    np.where(binned_pos == 0)[0], len(binned_pos)
+                )
+                # Get bin length of each chromosome
+                num_binned = binned_start[1:] - binned_start[:-1]
+                # Get unique chromosome names without losing original order
+                # (numpy.unique sorts output)
+                chr_names_idx = np.unique(
+                    self.frags.iloc[:, 1], return_index=True
+                )[1]
+                chr_names = [
+                    self.frags.iloc[index, 1]
+                    for index in sorted(chr_names_idx)
+                ]
+                binned_chrom = np.repeat(chr_names, num_binned)
+                binned_frags = pd.DataFrame(
+                    {"chrom": binned_chrom, "start_pos": binned_pos[:, 0]}
+                )
+                binned_frags["end_pos"] = binned_frags.groupby("chrom")[
+                    "start_pos"
+                ].shift(-1)
+                chrom_ends = self.frags.groupby("chrom").end_pos.max()
+                # Fill ends of chromosome bins with actual chromosome length
+                for cn in chrom_ends.index:
+                    binned_frags.end_pos[
+                        np.isnan(binned_frags.end_pos)
+                        & (binned_frags.chrom == cn)
+                    ] = chrom_ends[cn]
 
             else:
                 # Note this is a basic binning procedure, chromosomes are
@@ -417,8 +520,25 @@ class View(AbstractCommand):
                 binned_map = hcs.bin_sparse(
                     M=sparse_map, subsampling_factor=self.binning
                 )
+                binned_frags = self.frags.iloc[:: self.binning, :]
+                binned_frags = binned_frags.reset_index(drop=True)
+                # Since matrix binning ignores chromosomes, we
+                # have to do the same procedure with fragments
+                # we just correct the coordinates to start at 0
+                def shift_min(x):
+                    try:
+                        x[x == min(x)] = 0
+                    except ValueError:
+                        pass
+                    return x
+
+                binned_frags.start_pos = binned_frags.groupby(
+                    "chrom", sort=False
+                ).start_pos.apply(shift_min)
+
         else:
             binned_map = sparse_map
+            binned_frags = self.frags
 
         # TRIMMING
         if self.args["--trim"]:
@@ -440,6 +560,10 @@ class View(AbstractCommand):
 
         # ZOOM REGION
         if self.args["--region"]:
+            if self.args["--lines"]:
+                raise NotImplementedError(
+                    "Chromosome lines are currently incompatible with a region zoom"
+                )
             if self.frags is None:
                 logger.error(
                     "A fragment file must be provided to subset "
@@ -447,45 +571,7 @@ class View(AbstractCommand):
                 )
                 sys.exit(1)
             # Load chromosomes and positions from fragments list
-            reg_pos = self.frags.iloc[:, [1, 2]]
-            # Readjust bin coords post binning
-            if self.binning:
-                # Fixed genomic bins
-                if self.bp_unit:
-                    # Get bin numbers of chromosome starts
-                    binned_start = np.append(
-                        np.where(binned_frags == 0)[0], len(binned_frags)
-                    )
-                    # Get bin length of each chromosome
-                    num_binned = binned_start[1:] - binned_start[:-1]
-                    # Get unique chromosome names without losing original order
-                    # (numpy.unique sorts output)
-                    chr_names_idx = np.unique(
-                        reg_pos.iloc[:, 0], return_index=True
-                    )[1]
-                    chr_names = [
-                        reg_pos.iloc[index, 0]
-                        for index in sorted(chr_names_idx)
-                    ]
-                    binned_chrom = np.repeat(chr_names, num_binned)
-                    reg_pos = pd.DataFrame(
-                        {0: binned_chrom, 1: binned_frags[:, 0]}
-                    )
-                # Subsample binning (group by N frags)
-                else:
-                    reg_pos = reg_pos.iloc[:: self.binning, :]
-                    reg_pos = reg_pos.reset_index(drop=True)
-                    # Since matrix binning ignores chromosomes, we
-                    # have to do the same procedure with fragments
-                    # we just correct the coordinates to start at 0
-                    def shift_min(x):
-                        x[x == min(x)] = 0
-                        return x
-
-                    reg_pos.start_pos = reg_pos.groupby(
-                        "chrom", sort=False
-                    ).start_pos.apply(shift_min)
-
+            reg_pos = binned_frags.iloc[:, [0, 1]]
             region = self.args["--region"]
             if ";" in region:
                 # 2 input regions: zoom anywhere in matrix
@@ -501,7 +587,7 @@ class View(AbstractCommand):
             binned_map = binned_map[reg1[0] : reg1[1], reg2[0] : reg2[1]]
             binned_map = binned_map.tocoo()
 
-        return binned_map
+        return binned_map, binned_frags
 
     def execute(self):
 
@@ -551,7 +637,7 @@ class View(AbstractCommand):
             input_map, fragments_file=self.args["--frags"], quiet=True
         )
         output_file = self.args["--output"]
-        processed_map = self.process_matrix(sparse_map)
+        processed_map, frags = self.process_matrix(sparse_map)
         # If 2 matrices given compute log ratio
         if self.args["<contact_map2>"]:
             sparse_map2, _, _ = hio.flexible_hic_loader(
@@ -559,7 +645,7 @@ class View(AbstractCommand):
                 fragments_file=self.args["--frags"],
                 quiet=True,
             )
-            processed_map2 = self.process_matrix(sparse_map2)
+            processed_map2, _ = self.process_matrix(sparse_map2)
             if sparse_map2.shape != sparse_map.shape:
                 logger.error(
                     "You cannot compute the ratio of matrices with "
@@ -616,6 +702,11 @@ class View(AbstractCommand):
             else:
                 # Set 0 values in matrix to NA
                 dense_map[dense_map == 0] = np.inf
+            # Get chromosome coordinates if required
+            if self.args["--lines"]:
+                chrom_starts = np.where(np.diff(frags.start_pos) < 0)[0] + 1
+            else:
+                chrom_starts = None
             # Display NA values in white
             current_cmap = cm.get_cmap()
             current_cmap.set_bad(color=current_cmap(0))
@@ -627,6 +718,7 @@ class View(AbstractCommand):
                 vmax=self.vmax,
                 dpi=int(self.args["--dpi"]),
                 cmap=cmap,
+                chrom_starts=chrom_starts,
             )
         except MemoryError:
             logger.error("contact map is too large to load, try binning more")
@@ -640,10 +732,10 @@ class Pipeline(AbstractCommand):
 
     usage:
         pipeline [--aligner=bowtie2] [--centromeres=FILE] [--circular] [--distance-law]
-                 [--duplicates] [--enzyme=ENZ] [--filter] [--force] [--iterative] 
-                 [--matfmt=FMT] [--no-cleanup] [--outdir=DIR] [--plot] [--prefix=PREFIX]
-                 [--quality-min=INT] [--read-len=INT] [--remove-centromeres=INT] [--size=INT]
-                 [--start-stage=STAGE] [--threads=INT] [--tmpdir=DIR] --genome=FILE <input1> [<input2>]
+                 [--duplicates] [--enzyme=5000] [--filter] [--force] [--mapping=normal]
+                 [--matfmt=graal] [--no-cleanup] [--outdir=DIR] [--plot] [--prefix=PREFIX]
+                 [--quality-min=30] [--read-len=INT] [--remove-centromeres=0] [--size=0]
+                 [--start-stage=fastq] [--threads=1] [--tmpdir=DIR] --genome=FILE <input1> [<input2>]
 
     arguments:
         input1:             Forward fastq file, if start_stage is "fastq", sam
@@ -655,25 +747,25 @@ class Pipeline(AbstractCommand):
 
 
     options:
-        -a, --aligner=bowtie2         Alignment software to use. Can be either
+        -a, --aligner=STR             Alignment software to use. Can be either
                                       bowtie2, minimap2 or bwa. minimap2 should
-                                      only be used for reads > 100 bp. 
+                                      only be used for reads > 100 bp.
                                       [default: bowtie2]
         -c, --centromeres=FILE        Positions of the centromeres separated by
-                                      a space and in the same order than the 
+                                      a space and in the same order than the
                                       chromosomes. Discordant with the circular
                                       option.
-        -C, --circular                Enable if the genome is circular. 
-                                      Discordant with the centromeres option.   
+        -C, --circular                Enable if the genome is circular.
+                                      Discordant with the centromeres option.
         -d, --distance-law            If enabled, generates a distance law file
-                                      with the values of the probabilities to 
+                                      with the values of the probabilities to
                                       have a contact between two distances for
                                       each chromosomes or arms if the file with
                                       the positions has been given. The values
                                       are not normalized, or averaged.
         -D, --duplicates              Filter out PCR duplicates based on read
                                       positions.
-        -e, --enzyme=ENZ              Restriction enzyme if a string, or chunk
+        -e, --enzyme={STR|INT}        Restriction enzyme if a string, or chunk
                                       size (i.e. resolution) if a number. Can
                                       also be multiple comma-separated enzymes.
                                       [default: 5000]
@@ -683,15 +775,21 @@ class Pipeline(AbstractCommand):
                                       chunk size. For more informations, see
                                       Cournac et al. BMC Genomics, 2012.
         -F, --force                   Write even if the output file already exists.
-        -i, --iterative               Map reads iteratively using hicstuff
-                                      iteralign, by truncating reads to 20bp
-                                      and then repeatedly extending and
-                                      aligning them.
         -g, --genome=FILE             Reference genome to map against. Path to
-                                      the bowtie2/bwa index if using bowtie2/bwa, 
+                                      the bowtie2/bwa index if using bowtie2/bwa,
                                       or to a FASTA file if using minimap2.
-        -M, --matfmt=FMT              The format of the output sparse matrix.
-                                      Can be "bg2" for 2D Bedgraph format, 
+        -m, --mapping=STR             normal|iterative|cutsite. Parameter of 
+                                      mapping. "normal": Directly map reads 
+                                      without any process. "iterative": Map 
+                                      reads iteratively using iteralign, by 
+                                      truncating reads to 20bp and then 
+                                      repeatedly extending to align them.
+                                      "cutsite": Cut reads at the religation 
+                                      sites of the given enzyme using cutsite, 
+                                      create new pairs of reads and then align 
+                                      them ; enzyme is required [default: normal].
+        -M, --matfmt=STR              The format of the output sparse matrix.
+                                      Can be "bg2" for 2D Bedgraph format,
                                       "cool" for Mirnylab's cooler software, or
                                       "graal" for graal-compatible plain text
                                       COO format. [default: graal]
@@ -702,11 +800,11 @@ class Pipeline(AbstractCommand):
                                       directory.
         -p, --plot                    Generates plots in the output directory
                                       at different steps of the pipeline.
-        -P, --prefix=PREFIX           Overrides default filenames and prefixes all
+        -P, --prefix=STR              Overrides default filenames and prefixes all
                                       output files with a custom name.
         -q, --quality-min=INT         Minimum mapping quality for selecting
                                       contacts. [default: 30].
-        -r, --remove-centromeres=INT  Integer. Number of kb that will be remove around 
+        -r, --remove-centromeres=INT  Integer. Number of kb that will be remove around
                                       the centromere position given by in the centromere
                                       file. [default: 0]
         -R, --read-len=INT            Maximum read length in the fastq file. Optionally
@@ -716,12 +814,12 @@ class Pipeline(AbstractCommand):
         -s, --size=INT                Minimum size threshold to consider
                                       contigs. Keep all contigs by default.
                                       [default: 0]
-        -S, --start-stage=STAGE       Define the starting point of the pipeline
+        -S, --start-stage=STR         Define the starting point of the pipeline
                                       to skip some steps. Default is "fastq" to
                                       run from the start. Can also be "bam" to
                                       skip the alignment, "pairs" to start from a
                                       single pairs file or "pairs_idx" to skip
-                                      fragment attribution and only build the 
+                                      fragment attribution and only build the
                                       matrix. [default: fastq]
         -t, --threads=INT             Number of threads to allocate.
                                       [default: 1].
@@ -764,7 +862,7 @@ class Pipeline(AbstractCommand):
             enzyme=self.args["--enzyme"],
             filter_events=self.args["--filter"],
             force=self.args["--force"],
-            iterative=self.args["--iterative"],
+            mapping=self.args["--mapping"],
             mat_fmt=self.args["--matfmt"],
             min_qual=int(self.args["--quality-min"]),
             min_size=int(self.args["--size"]),
@@ -786,22 +884,22 @@ class Scalogram(AbstractCommand):
     Generate a scalogram.
 
     usage:
-        scalogram [--cmap=NAME] [--centromeres=FILE] [--frags=FILE] [--range=INT-INT]
-                  [--threads=INT] [--output=FILE] [--normalize]
+        scalogram [--cmap=viridis] [--centromeres=FILE] [--frags=FILE] [--range=INT-INT]
+                  [--threads=1] [--output=FILE] [--normalize]
                   [--indices=INT-INT] [--despeckle] <contact_map>
 
     argument:
         <contact_map> The sparse Hi-C contact matrix.
 
     options:
-        -C, --cmap=NAME                    The matplotlib colormap to use for
+        -C, --cmap=STR                     The matplotlib colormap to use for
                                            the plot. [default: viridis]
         -d, --despeckle                    Remove speckles (artifactual spots)
                                            from the matrix.
         -f, --frags=FILE                   Fragments_list.txt file providing mapping
                                            between genomic coordinates and bin IDs.
-        -i, --indices=INT-INT              The range of bin numbers of the matrix to 
-                                           use for the plot. Can also be given in 
+        -i, --indices=INT-INT              The range of bin numbers of the matrix to
+                                           use for the plot. Can also be given in
                                            UCSC style genomic coordinates (requires -f).
                                            E.g. chr1:1Mb-10Mb.
         -o, --output=FILE                  Output file where the plot should be
@@ -1053,13 +1151,16 @@ class Rebin(AbstractCommand):
         positions.reset_index(inplace=True)
         # Compute mean for all added features in each index bin
         # Normally only other feature is GC content
-        features = frags.agg("mean")
-        features.reset_index(inplace=True)
-        # set new bins positions
-        frags = features
-        frags["start_pos"] = 0
-        frags["end_pos"] = 0
-        frags.loc[:, positions.columns] = positions
+        try:
+            features = frags.agg("mean")
+            features.reset_index(inplace=True)
+            # set new bins positions
+            frags = features
+            frags["start_pos"] = 0
+            frags["end_pos"] = 0
+            frags.loc[:, positions.columns] = positions
+        except pd.core.base.DataError:
+            frags = positions
         frags["size"] = frags.end_pos - frags.start_pos
         cumul_bins = 0
         for chrom in chromnames:
@@ -1087,7 +1188,7 @@ class Subsample(AbstractCommand):
     Subsample contacts from a Hi-C matrix. Probability of sampling is proportional
     to the intensity of the bin.
     usage:
-        subsample  [--prop=FLOAT] [--force] <contact_map> <subsampled_prefix>
+        subsample  [--prop=0.1] [--force] <contact_map> <subsampled_prefix>
 
     arguments:
         contact_map             Sparse contact matrix in graal, bg2 or cool format.
@@ -1114,7 +1215,11 @@ class Subsample(AbstractCommand):
         subsampled = hcs.subsample_contacts(mat, float(self.args["--prop"]))
         subsampled = subsampled.tocoo()
         hio.flexible_hic_saver(
-            subsampled, prefix, frags=frags, hic_fmt=hic_fmt
+            subsampled,
+            prefix,
+            frags=frags,
+            hic_fmt=hic_fmt,
+            quiet=True,
         )
 
 
@@ -1124,8 +1229,8 @@ class Convert(AbstractCommand):
     bedgraph2D (bg2) and cooler (cool). Input format is automatically inferred.
 
     usage:
-        convert [--frags=FILE] [--chroms=FILE] [--force]
-                [--to=FORMAT] <contact_map> <prefix>
+        convert [--frags=FILE] [--chroms=FILE] [--force] [--genome=FILE]
+                [--to=cool] <contact_map> <prefix>
 
     arguments:
         contact_map               The file containing the contact frequencies.
@@ -1141,11 +1246,15 @@ class Convert(AbstractCommand):
                                   hicstuff pipeline. Required for graal
                                   matrices and recommended for bg2.
         -F, --force               Write even if the output file already exists.
+        -g, --genome=FILE         Optional genome file used to add a GC content
+                                  column to the fragments table. This is
+                                  required to generate instagraal-compatible
+                                  files.
         -c, --chroms=FILE         Tab-separated with headers, containing
                                   columns contig, length, n_frags, cumul_length.
                                   This is the file "info_contigs.txt" generated
                                   by hicstuff pipeline.
-        -T, --to=FORMAT           The format to which files should be
+        -T, --to=STR              The format to which files should be
                                   converted. [default: cool]
     """
 
@@ -1153,6 +1262,7 @@ class Convert(AbstractCommand):
         out_fmt = self.args["--to"]
         mat_path = self.args["<contact_map>"]
         frags_path = self.args["--frags"]
+        genome_path = self.args["--genome"]
         chroms_path = self.args["--chroms"]
         prefix = self.args["<prefix>"]
 
@@ -1167,7 +1277,20 @@ class Convert(AbstractCommand):
             chroms_file=chroms_path,
             quiet=True,
         )
+
+        # Modify fragments for instagraal compatibility
+        # Add fragments size column
+        chrom_col, start_col, end_col = hio.get_pos_cols(frags)
+        size = frags[end_col] - frags[start_col]
+        if "size" not in frags.columns:
+            frags = frags.join(pd.DataFrame({"size": size}))
+        # If genome was provided, add gc_content column
+        if genome_path:
+            gc = hio.gc_bins(genome_path, frags)
+            frags = frags.join(pd.DataFrame({"gc_content": gc}))
+
         # Write
+        mat = mat.astype(int)
         hio.flexible_hic_saver(
             mat=mat,
             out_prefix=prefix,
@@ -1181,58 +1304,58 @@ class Distancelaw(AbstractCommand):
     """Distance law tools.
     Take the distance law file from hicstuff and can average it, normalize it compute the
     slope of the curve and plot it.
-    
+
     usage:
-        distancelaw [--average] [--big-arm-only=INT] [--base=FLOAT] [--centromeres=FILE] 
-                    [--circular] [--frags=FILE] [--inf=INT] [--outputfile-img=IMG] 
-                    [--outputfile-tabl=TABLE] [--labels=DIR] [--sup=INT] 
-                    [--remove-centromeres=INT] (--pairs=FILE | --dist-tbl=FILE1[,FILE2,...])
-    
+        distancelaw [--average] [--big-arm-only=INT] [--base=1.1] [--centromeres=FILE]
+                    [--circular] [--frags=FILE] [--inf=INT] [--outputfile-img=FILE]
+                    [--outputfile-tabl=FILE] [--labels=DIR] [--sup=INT]
+                    [--remove-centromeres=0] (--pairs=FILE | --dist-tbl=FILE1[,FILE2,...])
+
     options:
-        -a, --average                       If set, calculate the average of the distance 
+        -a, --average                       If set, calculate the average of the distance
                                             law of the different chromosomes/arms in each
                                             condition. If two file given average is
                                             mandatory.
         -b, --big-arm-only=INT              Integer. It given will take only the arms bigger
                                             than the value given as argument.
-        -B, --base=FLOAT                    Float corresponding of the base of the log to 
-                                            make the logspace which will slice the genomes in
-                                            logbins. These slices will be in basepairs unit.
-                                            Default is 1.1.
+        -B, --base=FLOAT                    Float corresponding of the base of the log to
+                                            make the logspace which will slice the genomes
+                                            in logbins. These slices will be in basepairs
+                                            unit. [default is 1.1]
         -c, --centromeres=FILE              Positions of the centromeres separated by
-                                            a space and in the same order as the 
+                                            a space and in the same order as the
                                             chromosomes. This allows to plot chromosomal arms
                                             separately. Note this will only work with --pairs
                                             input, as the distance law needs to be recomputed.
                                             Incompatible with the circular option.
-        -C, --circular                      Enable if the genome is circular. Discordant 
+        -C, --circular                      Enable if the genome is circular. Discordant
                                             with the centromeres option.
-        -d, --dist-tbl=FILE1[,FILE2,...]    Directory to the file or files containing the 
+        -d, --dist-tbl=FILE1[,FILE2,...]    Directory to the file or files containing the
                                             compute distance law. File should have the same
                                             format than the ones made by hicstuff pipeline.
         -f, --frags=FILE                    Tab-separated file with headers, containing
                                             columns id, chrom, start_pos, end_pos size.
                                             This is the file "fragments_list.txt" generated by
-                                            hicstuff pipeline. Required if pairs are given. 
-        -i, --inf=INT                       Inferior born to plot the distance law. By 
+                                            hicstuff pipeline. Required if pairs are given.
+        -i, --inf=INT                       Inferior born to plot the distance law. By
                                             default the value is 3000 bp (3 kb). Have to
                                             be strictly positive.
-        -l, --labels=STR1,STR2...           List of string of the labels for the plot 
-                                            separated by a coma. If no labels given, give 
+        -l, --labels=STR1,STR2...           List of string of the labels for the plot
+                                            separated by a coma. If no labels given, give
                                             the names "Sample 1", "Sample 2"...
-        -o, --outputfile-img=IMG            Output file. Format must be compatible with 
+        -o, --outputfile-img=FILE           Output file. Format must be compatible with
                                             plt.savefig. Default : None.
         -O, --outputfile-tabl=TABLE         Output file. Default : None.
-        -p, --pairs=FILE                    Pairs file. Format from 4D Nucleome Omics Data 
-                                            Standards Working Group with the 8th and 9th 
-                                            coulumns are the ID of the fragments of the 
+        -p, --pairs=FILE                    Pairs file. Format from 4D Nucleome Omics Data
+                                            Standards Working Group with the 8th and 9th
+                                            coulumns are the ID of the fragments of the
                                             reads 1 and 2. Only add if no distance_law table
                                             given. It will compute the table from these pairs
                                             and the fragments from the fragments file.
-        -r, --remove-centromeres=INT        Integer. Number of kb that will be remove around 
+        -r, --remove-centromeres=INT        Integer. Number of kb that will be remove around
                                             the centromere position given by in the centromere
-                                            file. Default is zero. 
-        -s, --sup=INT                       Superior born to plot the distance law. By 
+                                            file. [default: 0]
+        -s, --sup=INT                       Superior born to plot the distance law. By
                                             default the value is the maximum length of all
                                             the dataset given. Also if big arm only set, it
                                             will be the minimum size of the arms/chromosomes
@@ -1260,13 +1383,10 @@ class Distancelaw(AbstractCommand):
                 output_file_tabl = self.args["--outputfile-tabl"]
             else:
                 output_file_tabl = None
-            # Check if centromeres file given and if the centromeres have to be remove
+            # Check if centromeres file given and if the centromeres have to be removed
             if self.args["--centromeres"]:
                 centromeres = self.args["--centromeres"]
-                if self.args["--remove-centromeres"]:
-                    rm_centro = int(self.args["--remove-centromeres"])
-                else:
-                    rm_centro = 0
+                rm_centro = int(self.args["--remove-centromeres"])
             else:
                 centromeres = None
                 rm_centro = 0
@@ -1277,7 +1397,7 @@ class Distancelaw(AbstractCommand):
                 circular = None
             # Check logarithm base
             if self.args["--base"]:
-                base = int(self.args["--base"])
+                base = float(self.args["--base"])
             else:
                 base = 1.1
             xs, ps, names = [None], [None], [None]
@@ -1377,7 +1497,7 @@ class Missview(AbstractCommand):
 
     options:
         -a, --aligner=STR    The read alignment software to use. Can be either
-                             bowtie2, minimap2 or bwa. minimap2 should only be 
+                             bowtie2, minimap2 or bwa. minimap2 should only be
                              used for reads > 100 bp. [default: bowtie2]
         -b, --binning=INT    Resolution to use to preview the Hi-C map. [default: 5000]
         -F, --force          Write even if the output file already exists.
